@@ -4,9 +4,9 @@
  
  nomacs is a fast and small image viewer with the capability of synchronizing multiple instances
  
- Copyright (C) 2011-2012 Markus Diem <markus@nomacs.org>
- Copyright (C) 2011-2012 Stefan Fiel <stefan@nomacs.org>
- Copyright (C) 2011-2012 Florian Kleber <florian@nomacs.org>
+ Copyright (C) 2011-2013 Markus Diem <markus@nomacs.org>
+ Copyright (C) 2011-2013 Stefan Fiel <stefan@nomacs.org>
+ Copyright (C) 2011-2013 Florian Kleber <florian@nomacs.org>
 
  This file is part of nomacs.
 
@@ -27,9 +27,9 @@
 
 #pragma once
 
-#ifdef WIN32		// why is Q_WS_WIN32 not defined here?
+#ifdef WIN32
 #include "shlwapi.h"
-//#pragma comment (lib, "shlwapi.lib")
+#pragma comment (lib, "shlwapi.lib")
 #endif
 
 #include <QtGui/QWidget>
@@ -55,18 +55,18 @@
 #include <QReadWriteLock>
 #include <QTimer>
 #include <QMovie>
+#include <QByteArray>
+#include <QCoreApplication>
 
-#ifdef HAVE_EXIV2_HPP
-#include <exiv2/exiv2.hpp>
-#else
-#include <exiv2/image.hpp>
-#include <iomanip>
+#ifdef WITH_WEBP
+#include "webp/decode.h"
+#include "webp/encode.h"
 #endif
 
 // opencv
 #ifdef WITH_OPENCV
 
-#ifdef Q_WS_WIN
+#ifdef WIN32
 #pragma warning(disable: 4996)
 #endif
 
@@ -81,13 +81,14 @@
 using namespace cv;
 #endif
 
+
 #include <set>
 
-#ifdef DK_DLL
-#define DllExport __declspec(dllexport)
-#else
-#define DllExport
-#endif
+//#ifdef DK_DLL
+//#define DllExport Q_DECL_EXPORT
+//#else
+//#define DllExport
+//#endif
 
 // TODO: ifdef
 //#include <ShObjIdl.h>
@@ -96,16 +97,39 @@ using namespace cv;
 
 // my classes
 //#include "DkNoMacs.h"
+#include "DkImageStorage.h"
 #include "DkTimer.h"
 #include "DkSettings.h"
+#include "DkThumbs.h"
+#include "DkMetaData.h"
+#include "../libqpsd/qpsdhandler.h"
 
 #ifdef Q_WS_X11
 	typedef  unsigned char byte;
 #endif
 
+#ifdef WITH_LIBTIFF
+	#ifdef Q_WS_WIN
+		#include "tif_config.h"	
+	#endif
+
+	#ifdef Q_WS_MAC
+		#define uint64 uint64_hack_
+		#define int64 int64_hack_
+	#endif // Q_WS_MAC
+
+	#include "tiffio.h"
+
+	#ifdef Q_WS_MAC
+		#undef uint64
+		#undef int64
+	#endif // Q_WS_MAC
+#endif
+
+
 namespace nmc {
 
-#ifdef WIN32
+#ifdef Q_WS_WIN
 	
 	/**
 	 * Logical string compare function.
@@ -124,493 +148,23 @@ namespace nmc {
 	bool wCompLogic(const std::wstring & lhs, const std::wstring & rhs);
 #endif
 
-bool wCompLogicQString(const QString & lhs, const QString & rhs);
 
+bool compLogicQString(const QString & lhs, const QString & rhs);
+
+bool compFilename(const QFileInfo & lhf, const QFileInfo & rhf);
+
+bool compFilenameInv(const QFileInfo & lhf, const QFileInfo & rhf);
+
+bool compDateCreated(const QFileInfo& lhf, const QFileInfo& rhf);
+
+bool compDateCreatedInv(const QFileInfo& lhf, const QFileInfo& rhf);
+
+bool compDateModified(const QFileInfo& lhf, const QFileInfo& rhf);
+
+bool compDateModifiedInv(const QFileInfo& lhf, const QFileInfo& rhf);
 
 // basic image processing
 
-/**
- * DkImage holds some basic image processing
- * methods that are generally needed.
- **/ 
-class DkImage {
-
-public:
-
-	/**< interpolation mapping OpenCV -> Qt */
-	enum{ipl_nearest, ipl_area, ipl_linear, ipl_cubic, ipl_lanczos, ipl_end};
-
-#ifdef WITH_OPENCV
-	
-	/**
-	 * Converts a QImage to a Mat
-	 * @param img formats supported: ARGB32 | RGB32 | RGB888 | Indexed8
-	 * @return cv::Mat the corresponding Mat
-	 **/ 
-	static Mat qImage2Mat(const QImage img) {
-
-		Mat mat2;
-		QImage cImg;	// must be initialized here!	(otherwise the data is lost before clone())
-
-		if (img.format() == QImage::Format_ARGB32 || img.format() == QImage::Format_RGB32 ) {
-			mat2 = Mat(img.height(), img.width(), CV_8UC4, (uchar*)img.bits(), img.bytesPerLine());
-			//qDebug() << "ARGB32 or RGB32";
-		}
-		else if (img.format() == QImage::Format_RGB888) {
-			mat2 = Mat(img.height(), img.width(), CV_8UC3, (uchar*)img.bits(), img.bytesPerLine());
-			//qDebug() << "RGB888";
-		}
-		else if (img.format() == QImage::Format_Indexed8) {
-			mat2 = Mat(img.height(), img.width(), CV_8UC1, (uchar*)img.bits(), img.bytesPerLine());
-			//qDebug() << "indexed...";
-		}
-		else {
-			//qDebug() << "image flag: " << img.format();
-			cImg = img.convertToFormat(QImage::Format_ARGB32);
-			mat2 = Mat(cImg.height(), cImg.width(), CV_8UC4, (uchar*)cImg.bits(), cImg.bytesPerLine());
-			//qDebug() << "I need to convert the QImage to ARGB32";
-		}
-
-		mat2 = mat2.clone();	// we need to own the pointer
-
-		return mat2; 
-	}
-
-	/**
-	 * Converts a cv::Mat to a QImage.
-	 * @param img supported formats CV8UC1 | CV_8UC3 | CV_8UC4
-	 * @return QImage the corresponding QImage
-	 **/ 
-	static QImage mat2QImage(Mat img) {
-
-		QImage qImg;
-
-		// since Mat header is copied, a new buffer should be allocated (check this!)
-		if (img.depth() == CV_32F)
-			img.convertTo(img, CV_8U, 255);
-
-		if (img.type() == CV_8UC1) {
-			qImg = QImage(img.data, (int)img.cols, (int)img.rows, (int)img.step, QImage::Format_Indexed8);	// opencv uses size_t if for scaling in x64 applications
-			//Mat tmp;
-			//cvtColor(img, tmp, CV_GRAY2RGB);	// Qt does not support writing to index8 images
-			//img = tmp;
-		}
-		if (img.type() == CV_8UC3) {
-			
-			//cv::cvtColor(img, img, CV_RGB2BGR);
-			qImg = QImage(img.data, (int)img.cols, (int)img.rows, (int)img.step, QImage::Format_RGB888);
-		}
-		if (img.type() == CV_8UC4) {
-			qImg = QImage(img.data, (int)img.cols, (int)img.rows, (int)img.step, QImage::Format_ARGB32);
-		}
-
-		qImg = qImg.copy();
-
-		return qImg;
-	}
-#endif
-
-	/**
-	 * Returns a string with the buffer size of an image.
-	 * @param img a QImage
-	 * @return QString a human readable string containing the buffer size
-	 **/ 
-	static QString getBufferSize(const QImage img) {
-
-		return getBufferSize(img.size(), img.depth());
-	}
-
-	/**
-	 * Returns a string with the buffer size of an image.
-	 * @param imgSize the image size
-	 * @param depth the image depth
-	 * @return QString a human readable string containing the buffer size
-	 **/ 
-	static QString getBufferSize(const QSize imgSize, const int depth) {
-
-		double size = (double)imgSize.width() * (double)imgSize.height() * (double)(depth/8.0f);
-		QString sizeStr;
-		qDebug() << "dimension: " << size;
-
-		if (size >= 1024*1024*1024) {
-			return QString::number(size/(1024.0f*1024.0f*1024.0f), 'f', 2) + " GB";
-		}
-		else if (size >= 1024*1024) {
-			return QString::number(size/(1024.0f*1024.0f), 'f', 2) + " MB";
-		}
-		else if (size >= 1024) {
-			return QString::number(size/1024.0f, 'f', 2) + " KB";
-		}
-		else {
-			return QString::number(size, 'f', 2) + " B";
-		}
-	}
-
-	/**
-	 * Returns a the buffer size of an image.
-	 * @param imgSize the image size
-	 * @param depth the image depth
-	 * @return buffer size in MB
-	 **/ 
-	static float getBufferSizeFloat(const QSize imgSize, const int depth) {
-
-		double size = (double)imgSize.width() * (double)imgSize.height() * (double)(depth/8.0f);
-		QString sizeStr;
-		//qDebug() << "dimension: " << size;
-
-		return size/(1024.0f*1024.0f);
-	}
-
-
-	/**
-	 * This function resizes an image according to the interpolation method specified.
-	 * @param img the image to resize
-	 * @param newSize the new size
-	 * @param factor the resize factor
-	 * @param interpolation the interpolation method
-	 * @return QImage the resized image
-	 **/ 
-	static QImage resizeImage(const QImage img, const QSize& newSize, float factor = 1.0f, int interpolation = ipl_cubic) {
-		
-		QSize nSize = newSize;
-
-		// nothing to do
-		if (img.size() == nSize && factor == 1.0f)
-			return img;
-
-		if (factor != 1.0f)
-			nSize = QSize(img.width()*factor, img.height()*factor);
-
-		if (nSize.width() < 1 || nSize.height() < 1) {
-			return QImage();
-		}
-
-		Qt::TransformationMode iplQt;
-		switch(interpolation) {
-		case ipl_nearest:	
-		case ipl_area:		iplQt = Qt::FastTransformation; break;
-		case ipl_linear:	
-		case ipl_cubic:		
-		case ipl_lanczos:	iplQt = Qt::SmoothTransformation; break;
-		}
-#ifdef WITH_OPENCV
-
-		int ipl = CV_INTER_CUBIC;
-		switch(interpolation) {
-		case ipl_nearest:	ipl = CV_INTER_NN; break;
-		case ipl_area:		ipl = CV_INTER_AREA; break;
-		case ipl_linear:	ipl = CV_INTER_LINEAR; break;
-		case ipl_cubic:		ipl = CV_INTER_CUBIC; break;
-#ifdef DISABLE_LANCZOS
-		case ipl_lanczos:	ipl = CV_INTER_CUBIC; break;
-#else
-		case ipl_lanczos:	ipl = CV_INTER_LANCZOS4; break;
-#endif
-		}
-
-
-		try {
-			Mat resizeImage = DkImage::qImage2Mat(img);
-
-			// is the image convertible?
-			if (resizeImage.empty()) {
-				return img.scaled(newSize, Qt::IgnoreAspectRatio, iplQt);
-			}
-			else {
-
-				Mat tmp;
-				cv::resize(resizeImage, tmp, cv::Size(nSize.width(), nSize.height()), 0, 0, ipl);
-				resizeImage = tmp;
-				return DkImage::mat2QImage(resizeImage);
-			}
-
-		}catch (std::exception se) {
-
-			return QImage();
-		}
-
-#else
-
-		return img.scaled(nSize, Qt::IgnoreAspectRatio, iplQt);
-
-#endif
-	}
-
-};
-
-
-class DllExport DkMetaData {
-
-public:
-	DkMetaData(QFileInfo file = QFileInfo()) {
-		this->file = file;
-		mdata = false;
-		hasMetaData = true;	// initially we assume that meta data is present
-		dirty = false;
-	};
-	
-	DkMetaData(const DkMetaData& metaData);
-
-	~DkMetaData() {};
-
-	DkMetaData& operator=(const DkMetaData& metadata) {
-
-		if (this == &metadata)
-			return *this;
-
-		this->file = metadata.file;
-		this->mdata = false;
-		this->hasMetaData = metadata.hasMetaData;
-		this->dirty = metadata.dirty;
-		
-		return *this;
-	};
-
-	void setFileName(QFileInfo file) {
-
-		// do nothing if the same file is set
-		if (this->file == file)
-			return;
-
-		this->file = file;
-		mdata = false;
-		hasMetaData = true;
-		dirty = false;
-	};
-
-	QFileInfo getFile() const {
-		return file;
-	};
-
-	bool isDirty() {
-		return dirty;
-	};
-
-	void reloadImg();
-
-	void saveMetaDataToFile(QFileInfo fileN = QFileInfo(), int orientation = 0);
-
-	std::string getNativeExifValue(std::string key);
-	std::string getExifValue(std::string key);
-	bool setExifValue(std::string key, std::string taginfo);
-	std::string getIptcValue(std::string key);
-	int getOrientation();
-	QImage getThumbnail();
-	void saveThumbnail(QImage thumb);
-	void saveOrientation(int o);
-	int getHorizontalFlipped();
-	void saveHorizontalFlipped(int f);
-	float getRating();
-	void saveRating(int r);
-	bool isTiff();
-	bool isJpg();
-	bool isRaw();
-	void printMetaData(); //only for debug
-	QStringList getExifKeys();
-	QStringList getExifValues();
-	QStringList getIptcKeys();
-	QStringList getIptcValues();
-
-
-private:
-
-	void readMetaData();
-
-	Exiv2::Image::AutoPtr exifImg;
-	QFileInfo file;
-
-	bool mdata;
-	bool hasMetaData;
-	bool dirty;
-
-};
-
-/**
- * This class holds thumbnails.
- **/ 
-class DkThumbNail {
-
-public:
-	enum {
-		exists_not = -1,
-		not_loaded,
-		loaded
-	};
-	
-	/**
-	 * Default constructor.
-	 * @param file the corresponding file
-	 * @param img the thumbnail image
-	 **/ 
-	DkThumbNail(QFileInfo file = QFileInfo(), QImage img = QImage()) {
-		this->img = img;
-		this->file = file;
-		imgExists = true;
-		s = qMax(img.width(), img.height());
-	};
-
-	/**
-	 * Default destructor.
-	 * @return 
-	 **/ 
-	~DkThumbNail() {};
-
-	/**
-	 * Sets the thumbnail image.
-	 * @param img the thumbnail
-	 **/ 
-	void setImage(QImage img) {
-		this->img = img;
-	}
-
-	/**
-	 * Returns the thumbnail.
-	 * @return QImage the thumbnail.
-	 **/ 
-	QImage getImage() {
-		return img;
-	};
-
-	/**
-	 * Returns the file information.
-	 * @return QFileInfo the thumbnail file
-	 **/ 
-	QFileInfo getFile() {
-		return file;
-	};
-
-	/**
-	 * Returns whether the thumbnail was loaded, or does not exist.
-	 * @return int a status (loaded | not loaded | exists not)
-	 **/ 
-	int hasImage() {
-		
-		if (!img.isNull())
-			return loaded;
-		else if (img.isNull() && imgExists)
-			return not_loaded;
-		else
-			return exists_not;
-	};
-
-	/**
-	 * Manipulates the file loaded status.
-	 * @param exists a status (loaded | not loaded | exists not)
-	 **/ 
-	void setImgExists(bool exists) {
-		imgExists = exists;
-	}
-
-	/**
-	 * Returns the thumbnail size.
-	 * @return int the maximal side (either width or height)
-	 **/ 
-	int size() {
-		return s;
-	};
-
-private:
-	QImage img;
-	QFileInfo file;
-	int s;
-	bool imgExists;
-
-};
-
-/**
- * This class provides a method for reading thumbnails.
- * If the a thumbnail is provided in the metadata,
- * it can be loaded very fast. Additionally,
- * the thumbnails are loaded in a separate thread (in the 
- * background)
- **/ 
-class DkThumbsLoader : public QThread {
-
-	Q_OBJECT
-
-public:
-	DkThumbsLoader(std::vector<DkThumbNail>* thumbs = 0, QDir dir = QDir(), QStringList files = QStringList());
-	~DkThumbsLoader() {};
-
-	void run();
-	int getFileIdx(QFileInfo& file);
-	QStringList getFiles() {
-		return files;
-	};
-	QDir getDir() {
-		return dir;
-	};
-	bool isWorking() {
-		return somethingTodo;
-	};
-
-	enum ForceUpdate {
-		not_forced,
-		dir_updated,
-		user_updated,
-	};
-
-signals:
-	void updateSignal();
-	void progressSignal(int percent);
-	void numFilesSignal(int numFiles);
-
-public slots:
-	void setLoadLimits(int start = 0, int end = 20);
-	void loadAll();
-	void stop();
-
-private:
-	std::vector<DkThumbNail>* thumbs;
-	QDir dir;
-	bool isActive;
-	bool somethingTodo;
-	int numFilesLoaded;
-	QMutex mutex;
-	int maxThumbSize;
-	int loadLimit;
-	int startIdx;
-	int endIdx;
-	bool loadAllThumbs;
-	bool forceSave;
-	QStringList files;
-
-	// function
-	QImage getThumbNailQt(QFileInfo file);
-	//QImage getThumbNailWin(QFileInfo file);
-	void init();
-	void loadThumbs();
-};
-
-class DkImageStorage : public QObject {
-	Q_OBJECT
-
-public:
-	DkImageStorage(QImage img = QImage());
-
-	void setImage(QImage img);
-	QImage getImage(float factor = 1.0f);
-	bool hasImage() {
-		return !img.isNull();
-	}
-
-public slots:
-	void computeImage();
-	void antiAliasingChanged(bool antiAliasing);
-
-signals:
-	void imageUpdated();
-
-protected:
-	QImage img;
-	QVector<QImage> imgs;
-
-	QMutex mutex;
-	QThread* computeThread;
-	bool busy;
-	bool stop;
-};
-
 /**
  * This class provides image loading and editing capabilities.
  * It additionally stores the currently loaded image.
@@ -626,13 +180,48 @@ public:
 		mode_end
 	};
 
+	enum loaderID {
+		no_loader = 0,
+		qt_loader,
+		psd_loader,
+		webp_loader,
+		raw_loader,
+		roh_loader,
+		hdr_loader,
+	};
+
 	DkBasicLoader(int mode = mode_default);
 
 	~DkBasicLoader() {
 		release();
 	};
 
-	bool loadGeneral(QFileInfo file);
+	/**
+	 * Loads the image for the given file
+	 * @param file an image file
+	 * @param skipIdx the number of (internal) pages to be skipped
+	 * @return bool true if the image was loaded
+	 **/ 
+	bool loadGeneral(QFileInfo file, bool rotateImg = false);
+
+	/**
+	 * Loads the page requested (with respect to the current page)
+	 * @param skipIdx number of pages to skip
+	 * @return bool true if we could load the page requested
+	 **/ 
+	bool loadPage(int skipIdx = 0);
+
+	int getNumPages() {
+		return numPages;
+	};
+
+	int getPageIdx() {
+		return pageIdx;
+	};
+
+	bool setPageIdx(int skipIdx);
+
+	bool save(QFileInfo fileInfo, QImage img, int compression = -1);
 	
 	/**
 	 * Sets a new image (if edited outside the basicLoader class)
@@ -643,7 +232,19 @@ public:
 
 		this->file = file;
 		qImg = img;
-	}
+	};
+
+	void setTraining(bool training) {
+		training = true;
+	};
+
+	bool getTraining() {
+		return training;
+	};
+
+	int getLoader() {
+		return loader;
+	};
 
 	/**
 	 * Returns the 8-bit image, which is rendered.
@@ -653,6 +254,14 @@ public:
 		return qImg;
 	};
 
+	QFileInfo getFile() {
+		return file;
+	};
+
+	bool isDirty() {
+		return pageIdxDirty;
+	};
+
 	/**
 	 * Returns the current image size.
 	 * @return QSize the image size.
@@ -676,6 +285,20 @@ public:
 	Mat getImageCv() { return cv::Mat(); };	// we should not need this
 #endif
 
+	bool loadPSDFile(QFileInfo fileInfo);
+#ifdef WITH_WEBP
+	bool loadWebPFile(QFileInfo fileInfo);
+	bool saveWebPFile(QFileInfo fileInfo, QImage img, int compression);
+	bool decodeWebP(const QByteArray& buffer);
+	bool encodeWebP(QByteArray& buffer, QImage img, int compression, int speed = 4);
+#else
+	bool loadWebPFile(QFileInfo fileInfo) {return false;};	// not supported if webP was not linked
+	bool saveWebPFile(QFileInfo fileInfo, QImage img, int compression) {return false;};
+	bool decodeWebP(const QByteArray& buffer) {return false;};
+	bool encodeWebP(QByteArray& buffer, QImage img, int compression, int speed = 4) {return false;};
+#endif
+
+
 public slots:
 	void rotate(int orientation);
 	void resize(QSize size, float factor = 1.0f, QImage* img = 0, int interpolation = DkImage::ipl_cubic, bool silent = false);
@@ -684,10 +307,17 @@ protected:
 	
 	bool loadRohFile(QString fileName);
 	bool loadRawFile(QFileInfo file);
+	void indexPages(const QFileInfo& fileInfo);
+	void convert32BitOrder(void *buffer, int width);
 
+	int loader;
+	bool training;
 	int mode;
 	QImage qImg;
 	QFileInfo file;
+	int numPages;
+	int pageIdx;
+	bool pageIdxDirty;
 
 #ifdef WITH_OPENCV
 	cv::Mat cvImg;
@@ -860,8 +490,8 @@ public:
 
 	virtual ~DkImageLoader();
 
-	static QString saveFilter;		// for system close dialog
-	static QString openFilter;		// for system  open dialog
+	//static QString saveFilter;		// for system close dialog
+	//static QString openFilter;		// for system  open dialog
 	static QStringList fileFilters;	// just the filters
 	static QStringList openFilters;	// for open dialog
 	static QStringList saveFilters;	// for close dialog
@@ -870,7 +500,7 @@ public:
 	QStringList keywords;
 	QStringList folderKeywords;		// are deleted if a new folder is opened
 
-	static bool isValid(QFileInfo& fileInfo);
+	static bool isValid(const QFileInfo& fileInfo);
 	//static int locateFile(QFileInfo& fileInfo, QDir* dir = 0);
 	static QStringList getFoldersRecursive(QDir dir);
 	static QStringList getFilteredFileList(QDir dir, QStringList ignoreKeywords = QStringList(), QStringList keywords = QStringList(), QStringList folderKeywords = QStringList());
@@ -897,7 +527,6 @@ public:
 	void setSaveDir(QDir& dir);
 	void setImage(QImage img, QFileInfo editFile = QFileInfo());
 	void load();
-	void load(QFileInfo file, bool silent = false, int cacheState = cache_default);
 	QImage loadThumb(QFileInfo& file, bool silent = false);
 	bool hasFile();
 	bool hasMovie();
@@ -905,7 +534,10 @@ public:
 	void updateCacheIndex();
 	QString fileName();
 	QFileInfo getChangedFileInfo(int skipIdx, bool silent = false, bool searchFile = true);
+	static QStringList sort(const QStringList& files, const QDir& dir);
+	void sort();
 
+	static void initFileFilters();	// add special file filters
 
 	/**
 	 * Returns if an image is loaded currently.
@@ -937,6 +569,10 @@ public:
 		return basicLoader.image();
 	};
 
+	bool dirtyTiff() {
+		return basicLoader.isDirty();
+	};
+
 	/**
 	 * Returns the image's metadata.
 	 * @return nmc::DkMetaData the image metadata.
@@ -953,7 +589,7 @@ signals:
 	void updateInfoSignal(QString msg, int time = 3000, int position = 0);
 	void updateInfoSignalDelayed(QString msg, bool start = false, int timeDelayed = 700);
 	void updateSpinnerSignalDelayed(bool start = false, int timeDelayed = 700);
-	void updateFileSignal(QFileInfo file, QSize s = QSize(), bool edited = false);
+	void updateFileSignal(QFileInfo file, QSize s = QSize(), bool edited = false, QString attr = QString());
 	void updateDirSignal(QFileInfo file, int force = DkThumbsLoader::not_forced);
 	void newErrorDialog(QString msg, QString title = "Error");
 	void fileNotLoadedSignal(QFileInfo file);
@@ -967,6 +603,7 @@ public slots:
 	void directoryChanged(const QString& path = QString());
 	void saveFileSilentIntern(QFileInfo file, QImage saveImg = QImage());
 	void saveFileIntern(QFileInfo filename, QString fileFilter = "", QImage saveImg = QImage(), int compression = -1);
+	void load(QFileInfo file, bool silent = false, int cacheState = cache_default);
 	virtual bool loadFile(QFileInfo file, bool silent = false, int cacheState = cache_default);
 	void saveRating(int rating);
 	void deleteFile();
@@ -1009,6 +646,7 @@ protected:
 	void updateHistory();
 	void startStopCacher();
 	void sendFileSignal();
+	QString getTitleAttributeString();
 };
 
 };
