@@ -4,9 +4,9 @@
 
  nomacs is a fast and small image viewer with the capability of synchronizing multiple instances
 
- Copyright (C) 2011-2012 Markus Diem <markus@nomacs.org>
- Copyright (C) 2011-2012 Stefan Fiel <stefan@nomacs.org>
- Copyright (C) 2011-2012 Florian Kleber <florian@nomacs.org>
+ Copyright (C) 2011-2013 Markus Diem <markus@nomacs.org>
+ Copyright (C) 2011-2013 Stefan Fiel <stefan@nomacs.org>
+ Copyright (C) 2011-2013 Florian Kleber <florian@nomacs.org>
 
  This file is part of nomacs.
 
@@ -68,9 +68,8 @@ DkImageManipulationDialog::~DkImageManipulationDialog() {
  **/
 void DkImageManipulationDialog::init() {
 
-	isOk = false;
 	dialogWidth = 700;
-	dialogHeight = 560;
+	dialogHeight = 600;
 	toolsWidth = 200;
 	previewMargin = 20;
 	previewWidth = dialogWidth - toolsWidth - 2 * previewMargin;
@@ -118,16 +117,6 @@ void DkImageManipulationDialog::createLayout() {
 	// bottom widget - buttons	
 	QWidget* bottomWidget = new QWidget(this);
 	QHBoxLayout* bottomWidgetHBoxLayout = new QHBoxLayout(bottomWidget);
-
-	QPushButton* buttonOk = new QPushButton(tr("&Ok"));
-	connect(buttonOk, SIGNAL(clicked()), this, SLOT(okPressed()));
-	QPushButton* buttonCancel = new QPushButton(tr("&Cancel"));
-	connect(buttonCancel, SIGNAL(clicked()), this, SLOT(cancelPressed()));
-
-	QSpacerItem* spacer = new QSpacerItem(1,1, QSizePolicy::Expanding, QSizePolicy::Expanding);
-	bottomWidgetHBoxLayout->addItem(spacer);
-	bottomWidgetHBoxLayout->addWidget(buttonOk);
-	bottomWidgetHBoxLayout->addWidget(buttonCancel);	
 	
 	// central widget - preview image
 	QWidget* centralWidget = new QWidget(this);
@@ -142,8 +131,8 @@ void DkImageManipulationDialog::createLayout() {
 	QVBoxLayout* toolsLayout = new QVBoxLayout(eastWidget);
 	toolsLayout->setContentsMargins(0,0,0,0);
 
-	contrastWidget = new DkContrast(eastWidget, this);
 	brightnessWidget = new DkBrightness(eastWidget, this);
+	contrastWidget = new DkContrast(eastWidget, this);
 	saturationWidget = new DkSaturation(eastWidget, this);
 	hueWidget = new DkHue(eastWidget, this);
 	gammaWidget = new DkGamma(eastWidget, this);
@@ -164,8 +153,16 @@ void DkImageManipulationDialog::createLayout() {
 
 	eastWidget->setLayout(toolsLayout);
 	
+	// buttons
+	QDialogButtonBox* buttons = new QDialogButtonBox(QDialogButtonBox::Ok | QDialogButtonBox::Cancel, Qt::Horizontal, this);
+	buttons->button(QDialogButtonBox::Ok)->setText(tr("&OK"));
+	buttons->button(QDialogButtonBox::Cancel)->setText(tr("&Cancel"));
+	buttons->setContentsMargins(10,10,10,10);
+	connect(buttons, SIGNAL(accepted()), this, SLOT(accept()));
+	connect(buttons, SIGNAL(rejected()), this, SLOT(reject()));
+
 	BorderLayout* borderLayout = new BorderLayout;
-	borderLayout->addWidget(bottomWidget, BorderLayout::South);
+	borderLayout->addWidget(buttons, BorderLayout::South);
 	borderLayout->addWidget(centralWidget, BorderLayout::Center);
 	borderLayout->addWidget(eastWidget, BorderLayout::East);
 	this->setSizeGripEnabled(false);
@@ -174,7 +171,7 @@ void DkImageManipulationDialog::createLayout() {
 }
 
 /**
-* rescale vieport image - it is used as a preview for the manipulation tools changes
+* rescale viewport image - it is used as a preview for the manipulation tools changes
  **/
 void DkImageManipulationDialog::createImgPreview() {
 
@@ -247,22 +244,6 @@ void DkImageManipulationDialog::drawImgPreview() {
 	previewLabel->setPixmap(QPixmap::fromImage(preview));
 }
 
-void DkImageManipulationDialog::okPressed() {
-
-	isOk = true;
-	this->close();
-}
-
-void DkImageManipulationDialog::cancelPressed() {
-
-	this->close();
-}
-
-void DkImageManipulationDialog::showEvent(QShowEvent *event) {
-	isOk = false;
-}
-
-
 /**
  * constructor for the abstract class DkImageManipulationWidget - all image manipulation widgets are created from it
  * @param parent widget
@@ -1614,4 +1595,4 @@ void DkUndoRedo::redoPressed() {
 
 
 
-};
\ No newline at end of file
+};
