@@ -4,9 +4,9 @@
  
  nomacs is a fast and small image viewer with the capability of synchronizing multiple instances
  
- Copyright (C) 2011-2012 Markus Diem <markus@nomacs.org>
- Copyright (C) 2011-2012 Stefan Fiel <stefan@nomacs.org>
- Copyright (C) 2011-2012 Florian Kleber <florian@nomacs.org>
+ Copyright (C) 2011-2013 Markus Diem <markus@nomacs.org>
+ Copyright (C) 2011-2013 Stefan Fiel <stefan@nomacs.org>
+ Copyright (C) 2011-2013 Florian Kleber <florian@nomacs.org>
 
  This file is part of nomacs.
 
@@ -50,6 +50,8 @@
 #include <QTimer>
 #include <QProcess>
 #include <QStringBuilder>
+#include <QDesktopWidget>
+#include <QProgressDialog>
 
 // OpenCV
 #ifdef WITH_OPENCV
@@ -74,25 +76,35 @@ using namespace cv;
 //	#pragma comment (lib, "dwmapi.lib")
 //#endif
 
-// my stuff
-#include "DkNetwork.h"
-#include "DkViewPort.h"
-#include "DkImage.h"
-#include "DkWidgets.h"
-#include "DkDialog.h"
-#include "DkSettings.h"
-#include "DkMenu.h"
-#include "DkTransferToolBar.h"
-#include "DkManipulationWidgets.h"
-
-#ifdef DK_DLL
-#define DllExport __declspec(dllexport)
-#else
-#define DllExport
-#endif
+#ifndef DllExport
+	#ifdef DK_DLL
+	#define DllExport Q_DECL_EXPORT
+	#else
+	#define DllExport Q_DECL_IMPORT
+	#endif
+#endif 
 
 namespace nmc {
 
+class DkTcpMenu;
+class DkCompressDialog;
+class DkTifDialog;
+class DkOpacityDialog;
+class DkOpenWithDialog;
+class DkResizeDialog;
+class DkUpdateDialog;
+class DkForceThumbDialog;
+class DkTrainDialog;
+class DkExplorer;
+class DkExportTiffDialog;
+class DkMosaicDialog;
+class DkImageManipulationDialog;
+class DkUpdater;
+class DkLocalManagerThread;
+class DkLanManagerThread;
+class DkTransferToolBar;
+
+
 // keyboard shortcuts
 //we can change the keyboard shortcuts from here !
 enum {	
@@ -101,9 +113,12 @@ enum {
 	shortcut_esc			= Qt::Key_Escape,
 	
 	// file
+	shortcut_show_scroller	= Qt::Key_F,
 	shortcut_open_preview	= Qt::Key_T,
+	shortcut_open_thumbview	= Qt::SHIFT + Qt::Key_T,
 	shortcut_open_dir		= Qt::CTRL + Qt::SHIFT + Qt::Key_O,
 	shortcut_open_with		= Qt::CTRL + Qt::Key_M,
+	shortcut_save_as		= Qt::CTRL + Qt::SHIFT + Qt::Key_S,
 	shortcut_first_file		= Qt::Key_Home, 
 	shortcut_last_file		= Qt::Key_End,
 	shortcut_skip_prev		= Qt::Key_PageUp,
@@ -133,15 +148,8 @@ enum {
 	shortcut_reset_view 	= Qt::CTRL + Qt::Key_0,
 	shortcut_zoom_full		= Qt::CTRL + Qt::Key_1,
 	shortcut_fit_frame		= Qt::CTRL + Qt::Key_2,
-	shortcut_zoom_in 		= Qt::Key_Plus,
-	shortcut_zoom_out		= Qt::Key_Minus,
-	shortcut_zoom_in_alt	= Qt::Key_Up,
-	shortcut_zoom_out_alt	= Qt::Key_Down,
-	shortcut_panning_left 	= Qt::CTRL + Qt::Key_Left,
-	shortcut_panning_right 	= Qt::CTRL + Qt::Key_Right,
-	shortcut_panning_up 	= Qt::CTRL + Qt::Key_Up,
-	shortcut_panning_down 	= Qt::CTRL + Qt::Key_Down,
 	shortcut_show_overview	= Qt::Key_O,
+	shortcut_show_explorer	= Qt::Key_E,
 	shortcut_show_player	= Qt::Key_P,
 	shortcut_show_exif		= Qt::Key_M,
 	shortcut_show_info		= Qt::Key_I,
@@ -151,6 +159,7 @@ enum {
 	shortcut_opacity_change	= Qt::ALT + Qt::SHIFT + Qt::Key_H,
 	shortcut_an_opacity		= Qt::ALT  + Qt::Key_H,
 	shortcut_new_instance	= Qt::CTRL + Qt::Key_N,
+	shortcut_tp_pattern		= Qt::Key_B,
 	shortcut_anti_aliasing	= Qt::Key_A,
 	shortcut_lock_window	= Qt::CTRL + Qt::SHIFT + Qt::ALT + Qt::Key_B,
 	//shortcut_play			= Qt::Key_Space,
@@ -166,6 +175,7 @@ enum {
 	shortcut_copy_buffer	= Qt::CTRL + Qt::SHIFT + Qt::Key_C,
 
 	// tcp
+	shortcut_shortcuts		= Qt::CTRL + Qt::Key_K,
 	shortcut_settings		= Qt::CTRL + Qt::SHIFT + Qt::Key_P,
 	shortcut_sync			= Qt::CTRL + Qt::Key_D,
 	shortcut_tab			= Qt::CTRL + Qt::Key_Tab,
@@ -188,6 +198,7 @@ enum fileActions {
 	menu_file_open_dir,
 	menu_file_open_with,
 	menu_file_save,
+	menu_file_save_as,
 	menu_file_rename,
 	menu_file_goto,
 	menu_file_find,
@@ -195,6 +206,7 @@ enum fileActions {
 	menu_file_reload,
 	menu_file_next,
 	menu_file_prev,
+	menu_file_train_format,
 	menu_file_new_instance,
 	menu_file_exit,
 	//menu_file_share_fb,
@@ -202,6 +214,17 @@ enum fileActions {
 	menu_file_end,	// nothing beyond this point
 };
 
+enum sortActions {
+
+	menu_sort_filename,
+	menu_sort_date_created,
+	menu_sort_date_modified,
+	menu_sort_ascending,
+	menu_sort_descending,
+
+	menu_sort_end,
+};
+
 enum editActions {
 	menu_edit_rotate_cw,
 	menu_edit_rotate_ccw,
@@ -209,8 +232,9 @@ enum editActions {
 	menu_edit_copy,
 	menu_edit_copy_buffer,
 	menu_edit_paste,
+	menu_edit_shortcuts,
 	menu_edit_preferences,
-	menu_edit_transfrom,
+	menu_edit_transform,
 	menu_edit_delete,
 	menu_edit_crop,
 	menu_edit_wallpaper,
@@ -222,15 +246,32 @@ enum toolsActions {
 	menu_tools_thumbs,
 	menu_tools_filter,
 	menu_tools_manipulation,
+	menu_tools_export_tiff,
+	menu_tools_mosaic,
 
 	menu_tools_end,
 };
 
+enum panelActions {
+	menu_panel_menu,
+	menu_panel_toolbar,
+	menu_panel_statusbar,
+	menu_panel_transfertoolbar,
+
+	menu_panel_player,
+	menu_panel_preview,
+	menu_panel_thumbview,
+	menu_panel_scroller,
+	menu_panel_exif,
+	menu_panel_info,
+	menu_panel_histogram,
+	menu_panel_overview,
+	menu_panel_explorer,
+
+	menu_panel_end,
+};
+
 enum viewActions {
-	menu_view_show_menu,
-	menu_view_show_toolbar,
-	menu_view_show_statusbar,
-	menu_view_show_transfertoolbar,
 	menu_view_fullscreen,
 	menu_view_reset,
 	menu_view_100,
@@ -238,12 +279,7 @@ enum viewActions {
 	menu_view_zoom_in,
 	menu_view_zoom_out,
 	menu_view_anti_aliasing,
-	menu_view_show_overview,
-	menu_view_show_player,
-	menu_view_show_preview,
-	menu_view_show_exif,
-	menu_view_show_info,
-	menu_view_show_histogram,
+	menu_view_tp_pattern,
 	menu_view_frameless,
 	menu_view_opacity_up,
 	menu_view_opacity_down,
@@ -251,6 +287,9 @@ enum viewActions {
 	menu_view_opacity_change,
 	menu_view_lock_window,
 	menu_view_gps_map,
+	menu_view_movie_pause,
+	menu_view_movie_next,
+	menu_view_movie_prev,
 
 	menu_view_end,	// nothing beyond this point
 };
@@ -264,37 +303,26 @@ enum syncActions {
 	menu_sync_end,	// nothing beyond this point
 };
 
+enum lanSyncActions {
+	menu_lan_server,
+	menu_lan_image,
+
+	menu_lan_end,
+};
+
 enum helpActions {
 	menu_help_update,
 	menu_help_bug,
 	menu_help_feature,
+	menu_help_documentation,
 	menu_help_about,
 	
 	menu_help_end,	// nothing beyond this point
 };
 
 enum shortcuts {
-	sc_pan_up,
-	sc_pan_down,
-	sc_pan_left,
-	sc_pan_right,
-	sc_first_file,
-	sc_last_file,
-	sc_skip_prev,
-	sc_skip_next,
 	sc_test_img,
 	sc_test_rec,
-	sc_next_sync,
-	sc_prev_sync,
-	sc_first_sync,
-	sc_last_sync,
-	sc_zoom_in,
-	sc_zoom_out,
-	sc_zoom_in_alt,
-	sc_zoom_out_alt,
-	//sc_send_img,
-	sc_delete_silent,
-	//sc_play,
 
 	sc_end,	// nothing beyond this point
 };
@@ -327,6 +355,9 @@ enum viewIcons {
 	icon_view_reset,
 	icon_view_100,
 	icon_view_gps,
+	icon_view_movie_play,
+	icon_view_movie_prev,
+	icon_view_movie_next,
 
 	icon_view_end,	// nothing beyond this point
 };
@@ -337,6 +368,15 @@ enum toolsIcons {
 	icon_tools_end,
 };
 
+enum statusbarLabels {
+	status_pixel_info,
+	status_filesize_info,
+	status_time_info,
+
+	status_end,
+
+};
+
 /*! A custom event filter for OSX.
 Its main purpose is to provide Mac OS X "open file from finder"
 functionality for nomacs. It *has* to be catched on the
@@ -378,8 +418,10 @@ public:
 	
 	QVector<QAction* > getFileActions();
 	QVector<QAction* > getBatchActions();
+	QVector<QAction* > getPanelActions();
 	QVector<QAction* > getViewActions();
 	QVector<QAction* > getSyncActions();
+	void loadFile(const QFileInfo& file, bool silent = false);
 
 	static void updateAll();
 
@@ -399,31 +441,42 @@ public slots:
 	void restart();
 	void enterFullScreen();
 	void exitFullScreen();
+	void openKeyboardShortcuts();
 	void openSettings();
+	void showExplorer(bool show);
 	void openDir();
 	void openFile();
 	void renameFile();
+	void changeSorting(bool change);
 	void goTo();
 	void find(bool filterAction = true);
 	void updateFilterState(QStringList filters);
 	void saveFile();
+	void saveFileAs(bool silent = false);
+	void trainFormat();
 	void resizeImage();
 	void openImgManipulationDialog();
+	void exportTiff();
+	void computeMosaic();
 	void deleteFile();
 	void setWallpaper();
+	void printDialog();
 	void cleanSettings();
 	void newInstance(QFileInfo file = QFileInfo());
 	void showStatusBar(bool show, bool permanent = true);
 	void showMenuBar(bool show);
+	void showToolbarsTemporarily(bool show);
 	void showToolbar(bool show);
+	void showToolbar(QToolBar* toolbar, bool show);
 	void showGpsCoordinates();
 	void openFileWith();
 	void aboutDialog();
+	void openDocumentation();
 	void bugReport();
 	void featureRequest();
 	void errorDialog(QString msg, QString title = "Error");
 	void loadRecursion();
-	void setWindowTitle(QFileInfo file, QSize size = QSize(), bool edited = false);
+	void setWindowTitle(QFileInfo file, QSize size = QSize(), bool edited = false, QString attr = QString());
 	void showOpacityDialog();
 	void opacityUp();
 	void opacityDown();
@@ -435,8 +488,8 @@ public slots:
 	void tcpSetWindowRect(QRect newRect, bool opacity, bool overlaid);
 	void tcpSendWindowRect();
 	void tcpSendArrange();
-	void newClientConnected();
-	void showStatusMessage(QString msg);
+	virtual void newClientConnected(bool connected, bool local);
+	void showStatusMessage(QString msg, int which = status_pixel_info);
 	void copyImage();
 	void copyImageBuffer();
 	void pasteImage();
@@ -451,6 +504,7 @@ public slots:
 	void setFrameless(bool frameless);
 	void fitFrame();
 	void setContrast(bool contrast);
+	void enableMovieActions(bool enable);
 	//void shareFacebook();
 
 	// batch actions
@@ -483,6 +537,7 @@ protected:
 	// TODO: put to android class
 	bool gestureEvent(QGestureEvent *event);
 
+	void assignCustomShortcuts(QVector<QAction*> actions);
 
 	bool otherKeyPressed;
 	QPoint posGrabKey;
@@ -493,10 +548,13 @@ protected:
 
 	QVector<QShortcut*> shortcuts;	
 	QVector<QAction *> fileActions;
+	QVector<QAction *> sortActions;
 	QVector<QAction *> editActions;
 	QVector<QAction *> toolsActions;
+	QVector<QAction *> panelActions;
 	QVector<QAction *> viewActions;
 	QVector<QAction *> syncActions;
+	QVector<QAction *> lanActions;
 	QVector<QAction *> helpActions;
 	//QVector<QAction *> tcpViewerActions;
 	
@@ -508,9 +566,11 @@ protected:
 
 	// menu
 	DkMenuBar* menu;
-	QMenu* fileMenu;	// TODO: release ?!
+	QMenu* fileMenu;
+	QMenu* sortMenu;
 	QMenu* editMenu;
 	QMenu* toolsMenu;
+	QMenu* panelMenu;
 	QMenu* viewMenu;
 	QMenu* syncMenu;
 	QMenu* helpMenu;
@@ -519,7 +579,7 @@ protected:
 	// sub menus
 	QMenu* fileFilesMenu;
 	QMenu* fileFoldersMenu;
-	QMenu* viewToolsMenu;
+	QMenu* panelToolsMenu;
 	DkTcpMenu* tcpViewerMenu;
 	DkTcpMenu* tcpLanMenu;
 	
@@ -527,19 +587,26 @@ protected:
 	
 	// toolbar
 	QToolBar* toolbar;
+	QToolBar* movieToolbar;
 	QStatusBar* statusbar;
-	QLabel* statusbarMsg;
+	QVector<QLabel*> statusbarLabels;
+	
 
 	// file dialog
 	QFileDialog* openDialog;
 	QFileDialog* saveDialog;
-	DkJpgDialog* jpgDialog;
+	DkCompressDialog* jpgDialog;
 	DkTifDialog* tifDialog;
 	DkOpacityDialog* opacityDialog;
 	DkOpenWithDialog* openWithDialog;
 	DkResizeDialog* resizeDialog;
 	DkUpdateDialog* updateDialog;
 	QProgressDialog* progressDialog;
+	DkForceThumbDialog* forceDialog;
+	DkTrainDialog* trainDialog;
+	DkExplorer* explorer;
+	DkExportTiffDialog* exportTiffDialog;
+	DkMosaicDialog* mosaicDialog;
 
 	DkImageManipulationDialog* imgManipulationDialog;
 
@@ -549,6 +616,7 @@ protected:
 	DkUpdater* updater;
 
 	QRect oldGeometry;
+	QList<QToolBar *> hiddenToolbars;
 
 	QProcess process;
 
@@ -586,6 +654,7 @@ public slots:
 	void tcpConnectAll();
 	void settingsChanged();
 	void clientInitialized();
+	void newClientConnected(bool connected, bool local);
 
 protected:
 
