@@ -4,9 +4,9 @@
  
  nomacs is a fast and small image viewer with the capability of synchronizing multiple instances
  
- Copyright (C) 2011-2012 Markus Diem <markus@nomacs.org>
- Copyright (C) 2011-2012 Stefan Fiel <stefan@nomacs.org>
- Copyright (C) 2011-2012 Florian Kleber <florian@nomacs.org>
+ Copyright (C) 2011-2013 Markus Diem <markus@nomacs.org>
+ Copyright (C) 2011-2013 Stefan Fiel <stefan@nomacs.org>
+ Copyright (C) 2011-2013 Florian Kleber <florian@nomacs.org>
 
  This file is part of nomacs.
 
@@ -49,6 +49,22 @@
 #include <QDesktopServices>
 #include <QVector2D>
 #include <qmath.h>
+#include <QScrollBar>
+#include <QPlastiqueStyle>
+#include <QFileSystemModel>
+#include <QDockWidget>
+#include <QTreeView>
+#include <QSortFilterProxyModel>
+#include <QToolTip>
+#include <QtConcurrentRun>
+#include <QProgressDialog>
+#include <QHeaderView>
+#include <QMenu>
+#include <QScrollArea>
+#include <QGraphicsView>
+#include <QGraphicsPixmapItem>
+#include <QGraphicsSceneMouseEvent>
+#include <QGraphicsItem>
 
 // gif animation label -----
 #include <QVBoxLayout>
@@ -60,6 +76,7 @@
 #include "DkNetwork.h"
 #include "DkSettings.h"
 #include "DkMath.h"
+#include "DkToolbars.h"
 
 #ifdef Q_WS_WIN
 #include <ShObjIdl.h>
@@ -67,6 +84,7 @@
 #endif
 
 namespace nmc {
+
 class DkThumbNail;
 
 class DkWidget : public QWidget {
@@ -93,12 +111,16 @@ public:
 		if (!displaySettingsBits)
 			return false;
 
-		if (DkSettings::App::currentAppMode < 0 || DkSettings::App::currentAppMode >= displaySettingsBits->size()) {
-			qDebug() << "[WARNING] illegal app mode: " << DkSettings::App::currentAppMode;
+		if (DkSettings::app.currentAppMode < 0 || DkSettings::app.currentAppMode >= displaySettingsBits->size()) {
+			qDebug() << "[WARNING] illegal app mode: " << DkSettings::app.currentAppMode;
 			return false;
 		}
 
-		return displaySettingsBits->testBit(DkSettings::App::currentAppMode);
+		return displaySettingsBits->testBit(DkSettings::app.currentAppMode);
+	};
+
+	bool isHiding() const {
+		return hiding;
 	};
 
 
@@ -127,7 +149,7 @@ protected:
 	void init();
 };
 
-class  DllExport DkLabel : public QLabel {
+class DllExport DkLabel : public QLabel {
 	Q_OBJECT
 
 public:
@@ -239,12 +261,12 @@ public:
 		if (!displaySettingsBits)
 			return false;
 
-		if (DkSettings::App::currentAppMode < 0 || DkSettings::App::currentAppMode >= displaySettingsBits->size()) {
-			qDebug() << "[WARNING] illegal app mode: " << DkSettings::App::currentAppMode;
+		if (DkSettings::app.currentAppMode < 0 || DkSettings::app.currentAppMode >= displaySettingsBits->size()) {
+			qDebug() << "[WARNING] illegal app mode: " << DkSettings::app.currentAppMode;
 			return false;
 		}
 
-		return displaySettingsBits->testBit(DkSettings::App::currentAppMode);
+		return displaySettingsBits->testBit(DkSettings::app.currentAppMode);
 	};
 
 signals:
@@ -410,7 +432,6 @@ protected:
 	virtual void paintEvent(QPaintEvent *event);
 };
 
-// TODO: check why it's not working with DkFadeLabel
 class DkFileInfoLabel : public DkFadeLabel {
 	Q_OBJECT
 
@@ -419,8 +440,8 @@ public:
 	~DkFileInfoLabel() {};
 
 	void createLayout();
-	void updateInfo(const QFileInfo& file, const QString& date, const int rating);
-	void updateTitle(const QFileInfo& file);
+	void updateInfo(const QFileInfo& file, const QString& attr, const QString& date, const int rating);
+	void updateTitle(const QFileInfo& file, const QString& attr);
 	void updateDate(const QString& date = QString());
 	void updateRating(const int rating);
 	void setEdited(bool edited);
@@ -490,7 +511,7 @@ public slots:
 
 	void startTimer() {
 		if (playing) {
-			displayTimer->setInterval(DkSettings::SlideShow::time*1000);	// if it was updated...
+			displayTimer->setInterval(DkSettings::slideShow.time*1000);	// if it was updated...
 			displayTimer->start();
 		}
 	};
@@ -532,16 +553,9 @@ class DkFilePreview : public DkWidget {
 	Q_OBJECT
 
 public:
-	DkFilePreview(QWidget* parent = 0, Qt::WFlags flags = 0);
+	DkFilePreview(DkThumbPool* thumbPool = 0, QWidget* parent = 0, Qt::WFlags flags = 0);
 	
 	~DkFilePreview() {
-		
-		if (thumbsLoader) {
-			thumbsLoader->stop();
-			thumbsLoader->wait();
-			delete thumbsLoader;
-			thumbsLoader = 0;
-		}
 	};
 
 	void setCurrentDx(float dx) {
@@ -553,13 +567,7 @@ public:
 		return moveImageTimer;
 	};
 
-	void setVisible(bool visible) {
-
-		DkWidget::setVisible(visible);
-
-		if (visible)
-			indexDir(DkThumbsLoader::not_forced);	// false = do not force refreshing the folder
-	}
+	void setVisible(bool visible);
 
 public slots:
 	void paintEvent(QPaintEvent *event);
@@ -570,17 +578,19 @@ public slots:
 	void wheelEvent(QWheelEvent *event);
 	void leaveEvent(QEvent *event);
 	void moveImages();
-	void updateDir(QFileInfo file, int force = DkThumbsLoader::not_forced);
+	void updateFileIdx(int fileIdx);
 
 signals:
 	void loadFileSignal(QFileInfo file);
-	void loadThumbsSignal(int start, int end);
+	//void loadThumbsSignal(int start, int end);
 	void changeFileSignal(int idx);
 
 private:
-	std::vector<DkThumbNail> thumbs;
-	DkThumbsLoader* thumbsLoader;
+	//QVector<QSharedPointer<DkThumbNailT>> thumbs;
+	DkThumbPool* thumbPool;
+	//DkThumbsLoader* thumbsLoader;
 	QDir thumbsDir;
+
 	
 	QWidget* parent;
 	QTransform worldMatrix;
@@ -606,7 +616,7 @@ private:
 	float currentDx;
 	QLabel* wheelButton;
 
-	DkGradientLabel* fileLabel;
+	//DkGradientLabel* fileLabel;
 
 	int selected;
 	float winPercent;
@@ -620,13 +630,230 @@ private:
 	bool scrollToCurrentImage;
 	
 	void init();
-	void indexDir(int force = DkThumbsLoader::not_forced);
+	//void clearThumbs();
+	//void indexDir(int force = DkThumbsLoader::not_forced);
 	void drawThumbs(QPainter* painter);
 	void drawFadeOut(QLinearGradient gradient, QRectF imgRect, QImage *img);
 	void createSelectedEffect(QImage img, QColor col);
 	void createCurrentImgEffect(QImage img, QColor col);
 };
 
+class DkThumbLabel : public QObject, public QGraphicsPixmapItem {
+	Q_OBJECT
+
+public:
+	DkThumbLabel(QSharedPointer<DkThumbNailT> thumb = QSharedPointer<DkThumbNailT>(), QGraphicsItem* parent = 0);
+
+	void setThumb(QSharedPointer<DkThumbNailT> thumb);
+	QSharedPointer<DkThumbNailT> getThumb() {return thumb;};
+	QRectF boundingRect() const;
+	void updateSize();
+
+public slots:
+	void updateLabel();
+
+signals:
+	void loadFileSignal(QFileInfo& file);
+	void showFileSignal(const QFileInfo& file);
+
+protected:
+	void mouseDoubleClickEvent(QGraphicsSceneMouseEvent *event);
+	void resizeEvent(QResizeEvent *event);
+	void paint(QPainter* painter, const QStyleOptionGraphicsItem* option, QWidget * widget = 0);
+	void hoverEnterEvent(QGraphicsSceneHoverEvent *event);
+	void hoverLeaveEvent(QGraphicsSceneHoverEvent *event);
+
+	QSharedPointer<DkThumbNailT> thumb;
+	QLabel* imgLabel;
+	bool thumbInitialized;
+	QPen noImagePen;
+	QBrush noImageBrush;
+	QPen selectPen;
+	QBrush selectBrush;
+	bool isHovered;
+};
+
+class DkThumbScene : public QGraphicsScene {
+	Q_OBJECT
+
+public:
+	DkThumbScene(DkThumbPool* thumbPool = 0, QWidget* parent = 0);
+
+	void updateLayout();
+	QList<QUrl> getSelectedUrls() const;
+	void setFile(const QFileInfo& file);
+
+public slots:
+	void updateThumbLabels();
+	void loadFile(QFileInfo& file);
+	void increaseThumbs();
+	void decreaseThumbs();
+	void resizeThumbs(int dx);
+	void showFile(const QFileInfo& file);
+	void selectThumbs(bool select = true, int from = 0, int to = -1);
+	void selectAllThumbs(bool select = true);
+
+signals:
+	void loadFileSignal(QFileInfo file);
+	void statusInfoSignal(QString msg, int pos = 0);
+
+protected:
+	//void wheelEvent(QWheelEvent *event);
+
+	DkThumbPool* thumbPool;
+	int xOffset;
+	int numRows;
+	int numCols;
+	bool firstLayout;
+	bool itemClicked;
+
+	QVector<QSharedPointer<DkThumbLabel> > thumbLabels;
+};
+
+class DkThumbsView : public QGraphicsView {
+	Q_OBJECT
+
+public:
+	DkThumbsView(DkThumbScene* scene, QWidget* parent = 0);
+
+protected:
+	void wheelEvent(QWheelEvent *event);
+	void dragEnterEvent(QDragEnterEvent *event);
+	void dropEvent(QDropEvent *event);
+	void dragMoveEvent(QDragMoveEvent *event);
+	void mousePressEvent(QMouseEvent *event);
+	void mouseMoveEvent(QMouseEvent *event);
+	void mouseReleaseEvent(QMouseEvent *event);
+
+	DkThumbScene* scene;
+	QPointF mousePos;
+
+};
+
+class DkThumbScrollWidget : public DkWidget {
+	Q_OBJECT
+
+public:
+	enum {
+		select_all,
+		zoom_in,
+		zoom_out,
+
+		actions_end
+	};
+
+	DkThumbScrollWidget(DkThumbPool* thumbPool = 0, QWidget* parent = 0, Qt::WindowFlags flags = 0);
+
+	DkThumbScene* getThumbWidget() {
+		return thumbsScene;
+	};
+
+public slots:
+	virtual void setVisible(bool visible);
+
+protected:
+	void createActions();
+	void resizeEvent(QResizeEvent *event);
+	void contextMenuEvent(QContextMenuEvent *event);
+
+	DkThumbScene* thumbsScene;
+	DkThumbPool* thumbPool;
+	DkThumbsView* view;
+
+	QMenu* contextMenu;
+	QVector<QAction*> actions;
+
+};
+
+class DkFolderScrollBar : public QScrollBar {
+	Q_OBJECT
+
+public: 
+	DkFolderScrollBar(QWidget* parent = 0);
+	~DkFolderScrollBar();
+
+	virtual void setValue(int i);
+
+	// DkWidget stuff
+	void registerAction(QAction* action) {
+		connect(this, SIGNAL(visibleSignal(bool)), action, SLOT(setChecked(bool)));
+	};
+
+	void block(bool blocked) {
+		this->blocked = blocked;
+		setVisible(false);
+	};
+
+	void setDisplaySettings(QBitArray* displayBits) {
+		displaySettingsBits = displayBits;
+	};
+
+	bool getCurrentDisplaySetting() {
+
+		if (!displaySettingsBits)
+			return false;
+
+		if (DkSettings::app.currentAppMode < 0 || DkSettings::app.currentAppMode >= displaySettingsBits->size()) {
+			qDebug() << "[WARNING] illegal app mode: " << DkSettings::app.currentAppMode;
+			return false;
+		}
+
+		return displaySettingsBits->testBit(DkSettings::app.currentAppMode);
+	};
+
+public slots:
+	void updateDir(QFileInfo file, int force = DkThumbsLoader::not_forced);
+	void update(const QVector<QColor>& colors, const QVector<int>& indexes);
+
+	// DkWidget
+	virtual void show();
+	virtual void hide();
+	virtual void setVisible(bool visible);
+
+	void animateOpacityUp();
+	void animateOpacityDown();
+
+protected slots:
+	void emitFileSignal(int i);
+
+signals:
+	void changeFileSignal(int idx);
+
+	// DkWidget
+	void visibleSignal(bool visible);
+
+protected:
+	void mousePressEvent(QMouseEvent *event);
+	void mouseMoveEvent(QMouseEvent *event);
+	void mouseReleaseEvent(QMouseEvent *event);
+	void resizeEvent(QResizeEvent *event);
+	
+	void indexDir(int force = DkThumbsLoader::not_forced);
+	
+	QDir currentDir;
+	QFileInfo currentFile;
+	QStringList files;
+	DkColorLoader* colorLoader;
+	QVector<QColor> colors;
+	bool sliding;
+	QLabel* handle;
+	DkWidget* dummyWidget;
+	int minHandleWidth;
+
+
+	// DkWidget
+	QColor bgCol;
+	bool blocked;
+	bool hiding;
+	bool showing;
+
+	QGraphicsOpacityEffect* opacityEffect;
+	QBitArray* displaySettingsBits;
+
+	void init();
+
+};
+
 // this class is one of the first batch processing classes -> move them to a new file in the (near) future
 class DkThumbsSaver : public DkWidget {
 	Q_OBJECT
@@ -634,20 +861,10 @@ class DkThumbsSaver : public DkWidget {
 public:
 	DkThumbsSaver() : thumbsLoader(0), pd(0) {};
 
-	void processDir(const QDir& dir);
+	void processDir(const QDir& dir, bool forceLoad);
 
 public slots:
-	void stopProgress() {
-		
-		if (pd)
-			pd->hide();
-		
-		if (thumbsLoader) {
-			thumbsLoader->stop();
-			thumbsLoader->wait();
-			delete thumbsLoader;
-		}
-	};
+	void stopProgress();
 
 protected:
 	std::vector<DkThumbNail> thumbs;
@@ -657,6 +874,57 @@ protected:
 	QProgressDialog* pd;
 };
 
+class DkFileSystemModel : public QFileSystemModel {
+	Q_OBJECT
+
+public:
+	DkFileSystemModel(QObject* parent = 0);
+
+protected:
+
+};
+
+class DkSortFileProxyModel : public QSortFilterProxyModel {
+	Q_OBJECT
+
+public:
+	DkSortFileProxyModel(QObject* parent = 0);
+
+protected:
+	virtual bool lessThan(const QModelIndex& left, const QModelIndex& right) const;
+
+};
+
+class DkExplorer : public QDockWidget {
+	Q_OBJECT
+
+public:
+	DkExplorer(const QString& title, QWidget* parent = 0, Qt::WindowFlags flags = 0);
+	~DkExplorer();
+
+public slots:
+	void setCurrentPath(QFileInfo fileInfo);
+	void fileClicked(const QModelIndex &index) const;
+	void showColumn(bool show);
+	void setEditable(bool editable);
+
+signals:
+	void openFile(QFileInfo fileInfo) const;
+	void openDir(QFileInfo dir) const;
+
+protected:
+	void closeEvent(QCloseEvent *event);
+	void contextMenuEvent(QContextMenuEvent* event);
+
+	void createLayout();
+	void writeSettings();
+	void readSettings();
+
+	DkFileSystemModel* fileModel;
+	DkSortFileProxyModel* sortModel;
+	QTreeView* fileTree;
+	QVector<QAction*> columnActions;
+};
 
 class DkOverview : public DkWidget {
 	Q_OBJECT
@@ -1006,6 +1274,21 @@ public:
 		return ((c2-c1)*0.5f + c1).getQPointF();
 	};
 
+	void setCenter(const QPointF& center) {
+
+		if (rect.empty())
+			return;
+
+		DkVector diff = getCenter() - center;
+
+		for (int idx = 0; idx < rect.size(); idx++) {
+
+			rect[idx] = rect[idx] - diff.getQPointF();
+		}
+
+
+	}
+
 	double getAngle() {
 		
 		// default upper left corner is 0
@@ -1042,8 +1325,6 @@ public:
 			size.setY(x);
 		}
 
-		//TODO: fix bug #132
-
 		// invariance -> user does not want to make a difference between an upside down rect
 		if (angle > CV_PI*0.25 && angle < CV_PI*0.75) {
 			angle -= CV_PI*0.5;
@@ -1094,7 +1375,7 @@ public:
 	};
 
 signals:
-	void ctrlMovedSignal(int, QPointF, bool);
+	void ctrlMovedSignal(int, QPointF, bool, bool);
 	void updateDiagonal(int);
 
 protected:
@@ -1145,22 +1426,34 @@ public:
 		this->imgRect = imgRect;
 	};
 
-	void setVisible(bool visible);
+	virtual void setVisible(bool visible);
 
 signals:
-	void enterPressedSignal(DkRotatingRect cropArea);
+	void enterPressedSignal(DkRotatingRect cropArea, const QColor& bgCol = QColor(0,0,0,0));
+	void angleSignal(double angle);
+	void aRatioSignal(const QPointF& aRatio);
+	void statusInfoSignal(QString msg);
 
 public slots:
-	void updateCorner(int idx, QPointF point, bool isShiftDown);
+	void updateCorner(int idx, QPointF point, bool isShiftDown, bool changeState = false);
 	void updateDiagonal(int idx);
+	void setFixedDiagonal(const DkVector& diag);
+	void setAngle(double angle, bool apply = true);
+	void setPanning(bool panning);
+	void setPaintHint(int paintMode = DkCropToolBar::no_guide);
+	void setShadingHint(bool invert);
+	void setShowInfo(bool showInfo);
 
 protected:
 	void mousePressEvent(QMouseEvent *event);
 	void mouseReleaseEvent(QMouseEvent *event);
 	void mouseMoveEvent(QMouseEvent *event);
+	void wheelEvent(QWheelEvent* event);
 	void keyPressEvent(QKeyEvent *event);
 	void keyReleaseEvent(QKeyEvent *event);
 	QPointF clipToImage(const QPointF& pos);
+	void applyTransform();
+	void drawGuide(QPainter* painter, const QPolygonF& p, int paintMode);
 	
 	void paintEvent(QPaintEvent *event);
 
@@ -1175,6 +1468,7 @@ protected:
 	QPointF posGrab;
 	QPointF clickPos;
 	DkVector oldDiag;
+	DkVector fixedDiag;
 
 	QWidget* parent;
 	DkRotatingRect rect;
@@ -1183,7 +1477,32 @@ protected:
 	QVector<DkTransformRect*> ctrlPoints;
 	QCursor rotatingCursor;
 	QRectF* imgRect;
+	bool panning;
+	int paintMode;
+	bool showInfo;
+};
 
+class DkCropWidget : public DkEditableRect {
+	Q_OBJECT
+
+public:
+	DkCropWidget(QRectF rect = QRect(), QWidget* parent = 0, Qt::WindowFlags f = 0);
+
+	virtual void setVisible(bool visible);
+	DkCropToolBar* getToolbar() {
+		return cropToolbar;
+	}
+
+public slots:
+	void crop();
+
+signals:
+	void showToolbar(QToolBar* toolbar, bool show);
+
+protected:
+	void createToolbar();
+
+	DkCropToolBar* cropToolbar;
 };
 
 /**
@@ -1310,6 +1629,11 @@ public:
 		return slider->value();
 	};
 
+	void setFocus(Qt::FocusReason reason) {
+		sliderBox->setFocus(reason);
+	};
+
+
 public slots:
 	void setValue(int value) {
 		
@@ -1338,4 +1662,4 @@ protected:
 	QSpinBox* sliderBox;
 };
 
-};
\ No newline at end of file
+};
